require 'dbots/err'
require 'dbots/ver'
require 'uri'
require 'json'
require 'rest-client'

module DBots
  # Main API functions
  module API
    module_function

    def get(url, headers = {})
      headers['User-Agent'] = user_agent
      request(:get, url, headers)
    end

    def post(url, body, headers = {})
      headers['User-Agent'] = user_agent
      request(:post, url, body, headers)
    end

    def user_agent
      "dbots (https://github.com/dbots-pkg/dbots.rb #{DBots::VERSION}) rest-client/#{RestClient::VERSION} #{RUBY_ENGINE}/#{RUBY_VERSION}p#{RUBY_PATCHLEVEL}"
    end

    # Request an endpoint easily
    def request(type, *attributes)
      response = RestClient.send(type, *attributes)
      Response.new(response, parse_json(response.body))
    rescue RestClient::Unauthorized
      raise DBots::Err::Unauthorized
    rescue RestClient::InternalServerError
      raise DBots::Err::ServerFail
    end

    # @!visibility private
    def parse_json(raw)
      JSON.parse(raw)
    rescue JSON::ParserError
      raw
    end

    # The nested response from rest-client
    class Response
      # @return [String] the HTTP response body.
      attr_reader :body

      # @return [Integer] HTTP status code.
      attr_reader :code
      alias_method :status, :code

      # @return [Hash] a hash of the headers, beautified with symbols and underscores.
      attr_reader :headers

      # @return [Hash] cookie jar extracted from response headers.
      attr_reader :cookies

      # @return [Hash, nil] the parsed JSON of the response
      attr_reader :json

      # @return [RestClient::Response] the raw response given from rest-client
      attr_reader :raw

      # @!visibility private
      def initialize(response, json = nil)
        @raw = response
        @code = response.code
        @headers = response.headers
        @cookies = response.cookies
        @json = json
      end

      # @!visibility private
      def inspect
        "<DBots::API::Response #{@code} #{@raw.body_truncated(10).inspect}>"
      end
    end
  end
end
