require 'dbots/api'
require 'dbots/err'
require 'uri'

# The main dbots module
module DBots
  # Represents a basic service.
  class Service
    BASE_URL = ''.freeze
    AUTH_HEADER = 'Authorization'.freeze

    # @return [String, nil] the token that will be used for the service
    attr_accessor :token
    alias_method :key, :token
    alias_method :api_key, :token

    # @param token [String] the token/key for the service.
    def initialize(token = nil)
      @token = token
    end

    def token?
      @token != nil
    end

    # @!visibility private
    def get(path, query = {}, headers = {})
      DBots::API.get("#{self.class::BASE_URL}#{path}?#{URI.encode_www_form(query)}", headers)
    end

    # @!visibility private
    def get_authed(path, query = {}, headers = {})
      raise DBots::Err::RequiresToken unless token?

      headers[self.class::AUTH_HEADER] = @token
      DBots::API.get("#{self.class::BASE_URL}#{path}?#{URI.encode_www_form(query)}", headers)
    end

    # @!visibility private
    def post_authed(path, body, query = {}, headers = {})
      raise DBots::Err::RequiresToken unless token?

      headers[self.class::AUTH_HEADER] = @token
      DBots::API.post("#{self.class::BASE_URL}#{path}?#{URI.encode_www_form(query)}", body, headers)
    end

    # @!visibility private
    def inspect
      "<#{self.class.name} @token?=#{token?}>"
    end
  end

  # Gets a service class based on a key
  # @param key [String] the service key to get
  # @returns [Service, nil]
  def self.get_service(key)
    services = DBots::Service.constants.select { |c| DBots::Service.const_get(c).is_a? Class }
    services.each do |service|
      service = DBots::Service.const_get(service)
      return service if service::ALIASES.include?(key.to_s)
    end

    nil
  end
end
