require 'dbots/data'

module DBots
  class Service
    # Represents the Discord Bots service.
    # @see https://discord.bots.gg/docs
    class DiscordBotsGG < Service
      BASE_URL = 'https://discord.bots.gg/api/v1'.freeze
      ALIASES = ['discordbotsgg', 'discord.bots.gg', 'botsgg', 'bots.gg', 'dbots'].freeze

      # Gets the bot listed on this service.
      # @Param id [String, Integer, #resolve_id] the bot's ID
      # @Param sanitied [true, false] whether to sanitize descriptions
      # @return [RestClient::Response]
      def get_bot(id, sanitied = false)
        id = id.respond_to?(:resolve_id) ? id.resolve_id.to_s : id.to_s
        get_authed("/bots/#{id}", { sanitied: sanitied })
      end

      # Gets a list of bots on this service.
      # @Param query [Hash] the query string that will be used in the request
      # @return [RestClient::Response]
      def get_bots(query = {})
        id = id.respond_to?(:resolve_id) ? id.resolve_id.to_s : id.to_s
        get_authed("/bots/#{id}", query)
      end
    end
  end
end
